@@ -23,12 +23,18 @@ sub generate_reset {
     if(ref $builder){
         return sub {
             my($instance) = @_;
+            if(@_ != 1) {
+                $self->argument_error('reset', 1, 1, scalar @_);
+            }
             $writer->($instance, $instance->$builder());
         };
     }
     else{
         return sub {
             my($instance) = @_;
+            if(@_ != 1) {
+                $self->argument_error('reset', 1, 1, scalar @_);
+            }
             $writer->($instance, $default);
         };
     }
@@ -37,7 +43,12 @@ sub generate_reset {
 sub generate_set{
     my($self)  = @_;
     my $writer = $self->writer;
-    return sub { $writer->( $_[0], $_[1] ) };
+    return sub {
+        if(@_ != 2) {
+            $self->argument_error('set', 2, 2, scalar @_);
+        }
+        $writer->( $_[0], $_[1] )
+    };
 }
 
 sub generate_inc {
@@ -49,11 +60,14 @@ sub generate_inc {
 
     return sub {
         my($instance, $value) = @_;
-        if(@_ > 1){
+        if(@_ == 1){
+            $value = 1;
+        }
+        elsif(@_ == 2){
             $constraint->assert_valid($value);
         }
-        else{
-            $value = 1;
+        else {
+            $self->argument_error('inc', 1, 2, scalar @_);
         }
         $writer->($instance, $reader->($instance) + $value);
     };
@@ -68,11 +82,14 @@ sub generate_dec {
 
     return sub {
         my($instance, $value) = @_;
-        if(@_ > 1){
+        if(@_ == 1){
+            $value = 1;
+        }
+        elsif(@_ == 2){
             $constraint->assert_valid($value);
         }
-        else{
-            $value = 1;
+        else {
+            $self->argument_error('dec', 1, 2, scalar @_);
         }
         $writer->($instance, $reader->($instance) - $value);
     };
